#include <iostream>
#include <fstream>
#include <cstring>
#include <sstream>
#include <string>
#include <vector>
#include <algorithm>
#include <iomanip>
#include <tuple>
using namespace std;

// stackoverflow.com/questions/236129
void split(vector<string>& tokens, const string &text, char sep)
{
	int start = 0, end = 0;
	while ((end = text.find(sep, start)) != string::npos) {
		tokens.push_back(text.substr(start, end-start));
		start = end + 1;
	}
	tokens.push_back(text.substr(start));
}
unsigned short storeOptions(bool options[])
{
	unsigned short opt = 0;
	for (int i = 15; i >= 0; i--)
	{
		opt = (opt << 1) + (options[i] == true ? 1 : 0);
	}
	return opt;
}
int stringToInt(string& s, string dot=".", short precision=2)
{
	string line = s;
	size_t z = line.find(dot);
	int decimal_position = 0;
	if (z != string::npos)
		decimal_position = line.length() - z - 1;
	line.erase(remove_if(line.begin(), line.end(), 
		[&] (char c) -> bool { return c==dot[0]; }), line.end());
	int k = atoi(line.c_str()) * pow(10, precision-decimal_position);

	return k;
}
float stringToFloat(string& s)
{
	char z[50];
	strcpy(z, s.c_str());
	char* z2 = z;
	while(*(++z2)) if(*z2 == ',') *z2 = '.';

	float f;
	sscanf(z, "%f", &f);
	return f;
}
double stringToDouble(string& s)
{
	char z[50];
	strcpy(z, s.c_str());
	char* z2 = z;
	while(*(++z2)) if(*z2 == ',') *z2 = '.';

	double f;
	sscanf(z, "%lf", &f);
	return f;
}

int main(int argc, char*argv[])
{
	if (argc < 2)
	{
		cout << "Requires argument(s)!" << endl;
		return 0;
	}
	ifstream inputData;
	inputData.open(argv[1]);
	if (strstr(argv[1], "curve1.dat") != NULL)
	{
		cout.put(1);

		string line;
		int i = 0;
		int last = 0;
		// http://stackoverflow.com/questions/5891610/how-to-remove-characters-from-a-string
		while (getline(inputData, line))
		{
			i++;
			line.erase(remove_if(line.begin(), line.end(),
				   [] (char c) -> bool { return c == '.'; }), line.end());
			int k = atoi(line.c_str());
			int16_t j = k - last;
			last = k;
			cout.write(reinterpret_cast<const char*>(&j), sizeof(j));
		}
	}
	else if (strstr(argv[1], "paleo.csv") != NULL)
	{
		cout.put(2);

		string line;
		vector<string> names;
		names.push_back("");
		struct paleoRecord {
			bool options[32] = { false };;
			vector<string>* tokens;
			unsigned short X1, maxdm, dm, ww, wh, uw, ah, maxdm2;
			char lobes;
			string WER;
			string record;
		};
		vector<paleoRecord> records;
		bool first = true;
		while (getline(inputData, line))
		{
			paleoRecord p;
			bool* options = p.options;
			if (first) { options[15] = true; first = false; }
			p.tokens = new vector<string>();
			vector<string>& tokens = *p.tokens;
			split(tokens, line, ';');
			for (int i = 0; i < 14; i++)
			{	if (i < 2 || i > 6)
				{
					if (tokens[i].length() > 0) options[i] = true;
				}
				else
					if (tokens[i].length() == 0) options[15] = true;
			}
			if (options[11] && tokens[11][0] == '#') options[14] = true;

			if (tokens[0] == names.back()) options[0] = false;
			else names.push_back(tokens[0]);

			p.record = line;
			if (options[15]) { records.push_back(p); continue; }
			
			if (options[1]) p.X1 = atoi(tokens[1].c_str());
			p.maxdm = atoi(tokens[2].c_str());
			p.dm = stringToInt(tokens[3], ",");
			p.ww = stringToInt(tokens[4], ",");
			p.wh = stringToInt(tokens[5], ",");
			p.uw = stringToInt(tokens[6], ",");
			if (options[7]) p.ah = stringToInt(tokens[7], ",");
			if (options[7] && p.ah == 0) options[3] = true;
			if (options[8]) p.lobes = atoi(tokens[8].c_str());
			if (options[9]) p.maxdm2 = stringToInt(tokens[9], ",");
			if (options[9] && p.maxdm*100 == p.maxdm2) options[2] = true;
			
			char buf[50];
			if (options[10])
			{
				sprintf(buf, "%.9g", 1.0*p.ww/p.dm);
				char *z = buf;
				while(*(++z)){if(*z=='.')*z=',';}
				if (tokens[10] != string(buf))
					options[4] = true;
			}
			if (options[11] && !options[14])
			{
				sprintf(buf, "%.9g", 1.0*p.ww/p.wh);
				char *z = buf;
				while(*(++z)){if(*z=='.')*z=',';}
				if (tokens[11] != string(buf))
					options[5] = true;
			}
			if (options[12])
			{
				sprintf(buf, "%.9g", 1.0*p.uw/p.dm);
				char *z = buf;
				while(*(++z)){if(*z=='.')*z=',';}
				if (tokens[12] != string(buf))
					options[6] = true;
			}

			records.push_back(p);
		}
		short names_length = names.size();
		cout.write(reinterpret_cast<const char*>(&names_length), 2);
		
		for (auto &i : names)
		{
			cout << i;
			cout.put(0);
		}
		short records_length = records.size();
		cout.write(reinterpret_cast<const char*>(&records_length), 2);

		int i = 0;
		int optstats[16] = { 0 };
		for (auto &p : records)
		{
			bool* options = p.options;
			unsigned short storedOptions = storeOptions(options);

			for (int j = 0; j < 16; j++)
				if (options[j]) optstats[j]++;
			cout.write(reinterpret_cast<const char*>(&storedOptions), 2);

			if (options[15])
			{
				cout << p.record; cout.put(0); continue;
			}
			if (options[1])
				cout.write(reinterpret_cast<const char*>(&p.X1), 2);
			
			cout.write(reinterpret_cast<const char*>(&p.maxdm), 2);
			cout.write(reinterpret_cast<const char*>(&p.dm), 2);
			cout.write(reinterpret_cast<const char*>(&p.ww), 2);
			cout.write(reinterpret_cast<const char*>(&p.wh), 2);
			cout.write(reinterpret_cast<const char*>(&p.uw), 2);

			if (options[7] && !options[3])
				cout.write(reinterpret_cast<const char*>(&p.ah), 2);
			if (options[8])
				cout.write(reinterpret_cast<const char*>(&p.lobes), 1);
			if (options[9] && !options[2])
				cout.write(reinterpret_cast<const char*>(&p.maxdm2), 2);
			if (options[10] && options[4]) {
				cout << (*p.tokens)[10];
				cout.put(0);
				//double f = stringToDouble((*p.tokens)[10]);
				//cout.write(reinterpret_cast<const char*>(&f), 8);
			}
			if (options[11] && options[5] && !options[14]) {
				cout << (*p.tokens)[11];
				cout.put(0);
				//double f = stringToDouble((*p.tokens)[11]);
				//cout.write(reinterpret_cast<const char*>(&f), 8);
			}
			if (options[12] && options[6]) {
				cout << (*p.tokens)[12];
				cout.put(0);
				//double f = stringToDouble((*p.tokens)[12]);
				//cout.write(reinterpret_cast<const char*>(&f), 8);
			}
			if (options[13]) {
				cout << (*p.tokens)[13];
				cout.put(0);
				//double f = stringToDouble((*p.tokens)[13]);
				//cout.write(reinterpret_cast<const char*>(&f), 8);
			}
		}
		/*cerr << endl << "OPTSTATS:" << endl;
		for (int j = 0; j < 16; j++)
		{
			cerr << optstats[j] << ";";
		} cerr << endl;*/
	}
	else if (strstr(argv[1], "ty.txt") != NULL)
	{
		cout.put(3);

		string line;
		int16_t i = 0;
		int last = 0;
		vector<tuple<int16_t,int>> insertions;
		vector<int16_t> diffs;
		// http://stackoverflow.com/questions/5891610/how-to-remove-characters-from-a-string
		while (getline(inputData, line))
		{
			int k = stringToInt(line);
			if ((k - last) > 32767 || (k - last) < -32768)
			{
				insertions.push_back(make_tuple(i, k));
				i++;
				continue;
			}
			int16_t j = k - last;
			last = k;
			diffs.push_back(j);
			i++;
		}
		short insertions_size = insertions.size();
		cout.write(reinterpret_cast<const char*>(&insertions_size), 2);
		for (auto &i : insertions)
		{
			cout.write(reinterpret_cast<const char*>(&get<0>(i)), 2);
			cout.write(reinterpret_cast<const char*>(&get<1>(i)), 4);
		}
		for (auto &i : diffs)
		{
			cout.write(reinterpret_cast<const char*>(&i), 2);
		}
	}
	else if (strstr(argv[1], "final.dat") != NULL)
	{
		cout.put(4);
		
		ifstream helpData;
		helpData.open(argv[2]);
		
		double c1 = 2.814564, c2 = -4.07616, ci = -2.503692;
		cout.write(reinterpret_cast<const char*>(&c1), 8);
		cout.write(reinterpret_cast<const char*>(&c2), 8);
		cout.write(reinterpret_cast<const char*>(&ci), 8);
		string line, helpLine;
		int fdf = 0;
		while (getline(inputData, line))
		{
			double a1; helpData >> a1;
			double a2; helpData >> a2;
			double f = stringToDouble(line);
			double q = f - ci - c1 * a1 - c2 * a2;
			q = (q + 40) * 1000;
			unsigned short v = (unsigned short)(q+0.5);
			cout.write(reinterpret_cast<const char*>(&v), 2);
		}
	}
	else
	{
		cout.put(0);

		char k;
		while(true)
		{
			inputData.get(k);
			if (inputData.eof()) break;
			cout.put(k);
		}
	}

	return 0;
}
